from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

class RatingValidator(object):
    def __init__(self, max):
        self.max = max

    def __call__(self, value):
        try:
            rating = int(value)
        except (TypeError, ValueError):
            raise ValidationError(
                _('%ss is not a number.' % value)
            )

        if rating > self.max:
            raise ValidationError(
                _('Value cannot be greater than maximum allowed number of ratings.')
            )

        if rating < 1:
            raise ValidationError(
                _('Value cannot be less than 1.')
            )





from django import forms
from django.utils.translation import gettext_lazy as _
from djf_surveys.models import Question, Survey
from djf_surveys.widgets import InlineChoiceField


class QuestionForm(forms.ModelForm):
    
    class Meta:
        model = Question
        fields = ['label', 'key', 'help_text', 'required']


class QuestionWithChoicesForm(forms.ModelForm):
    
    class Meta:
        model = Question
        fields = ['label', 'key', 'choices', 'help_text', 'required']
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['choices'].widget = InlineChoiceField()
        self.fields['choices'].help_text = _("Click Button Add to adding choice")


class QuestionFormRatings(forms.ModelForm):
    
    class Meta:
        model = Question
        fields = ['label', 'key', 'choices', 'help_text', 'required']
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['choices'].widget = forms.NumberInput(attrs={'max':10, 'min':1})
        self.fields['choices'].help_text = _("Must be between 1 and 10")
        self.fields['choices'].label = _("Number of ratings")
        self.fields['choices'].initial = 5


class SurveyForm(forms.ModelForm):
    
    class Meta:
        model = Survey
        fields = [
            'name', 'description', 'editable', 'deletable', 
            'duplicate_entry', 'private_response', 'can_anonymous_user',
            'notification_to', 'success_page_content'
        ]
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['notification_to'].widget = InlineChoiceField()



# Generated by Django 4.1.3 on 2022-11-05 22:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("djf_surveys", "0009_auto_20220803_0927"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="answer",
            options={
                "ordering": ["question__ordering"],
                "verbose_name": "answer",
                "verbose_name_plural": "answers",
            },
        ),
        migrations.AlterModelOptions(
            name="question",
            options={
                "ordering": ["ordering"],
                "verbose_name": "question",
                "verbose_name_plural": "questions",
            },
        ),
        migrations.AlterModelOptions(
            name="survey",
            options={"verbose_name": "survey", "verbose_name_plural": "surveys"},
        ),
        migrations.AlterModelOptions(
            name="useranswer",
            options={
                "ordering": ["-updated_at"],
                "verbose_name": "user answer",
                "verbose_name_plural": "user answers",
            },
        ),
        migrations.AlterField(
            model_name="answer",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="answer",
            name="question",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="answers",
                to="djf_surveys.question",
                verbose_name="question",
            ),
        ),
        migrations.AlterField(
            model_name="answer",
            name="user_answer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="djf_surveys.useranswer",
                verbose_name="user answer",
            ),
        ),
        migrations.AlterField(
            model_name="answer",
            name="value",
            field=models.TextField(
                help_text="The value of the answer given by the user.",
                verbose_name="value",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="choices",
            field=models.TextField(
                blank=True,
                help_text="If type of field is radio, select, or multi select, fill in the options separated by commas. Ex: Male, Female.",
                null=True,
                verbose_name="choices",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="help_text",
            field=models.CharField(
                blank=True,
                help_text="You can add a help text in here.",
                max_length=200,
                null=True,
                verbose_name="help text",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="key",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Unique key for this question, fill in the blank if you want to use for automatic generation.",
                max_length=225,
                unique=True,
                verbose_name="key",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="label",
            field=models.CharField(
                help_text="Enter your question in here.",
                max_length=500,
                verbose_name="label",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="ordering",
            field=models.PositiveIntegerField(
                default=0,
                help_text="Defines the question order within the surveys.",
                verbose_name="choices",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="required",
            field=models.BooleanField(
                default=True,
                help_text="If True, the user must provide an answer to this question.",
                verbose_name="required",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="survey",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="questions",
                to="djf_surveys.survey",
                verbose_name="survey",
            ),
        ),
        migrations.AlterField(
            model_name="question",
            name="type_field",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "Text"),
                    (1, "Number"),
                    (2, "Radio"),
                    (3, "Select"),
                    (4, "Multi Select"),
                    (5, "Text Area"),
                    (6, "URL"),
                    (7, "Email"),
                    (8, "Date"),
                    (9, "Rating"),
                ],
                verbose_name="type of input field",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="can_anonymous_user",
            field=models.BooleanField(
                default=False,
                help_text="If True, user without authentatication can submit.",
                verbose_name="anonymous submission",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="deletable",
            field=models.BooleanField(
                default=True,
                help_text="If False, user can't delete record.",
                verbose_name="deletable",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="description",
            field=models.TextField(default="", verbose_name="description"),
        ),
        migrations.AlterField(
            model_name="survey",
            name="duplicate_entry",
            field=models.BooleanField(
                default=False,
                help_text="If True, user can resubmit.",
                verbose_name="mutiple submissions",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="editable",
            field=models.BooleanField(
                default=True,
                help_text="If False, user can't edit record.",
                verbose_name="editable",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="name",
            field=models.CharField(max_length=200, verbose_name="name"),
        ),
        migrations.AlterField(
            model_name="survey",
            name="private_response",
            field=models.BooleanField(
                default=False,
                help_text="If True, only admin and owner can access.",
                verbose_name="private response",
            ),
        ),
        migrations.AlterField(
            model_name="survey",
            name="slug",
            field=models.SlugField(default="", max_length=225, verbose_name="slug"),
        ),
        migrations.AlterField(
            model_name="useranswer",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="useranswer",
            name="survey",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="djf_surveys.survey",
                verbose_name="survey",
            ),
        ),
        migrations.AlterField(
            model_name="useranswer",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
    ]



import csv
from io import StringIO

from django.utils.text import capfirst
from django.utils.translation import gettext, gettext_lazy as _
from django.views.generic.edit import CreateView, UpdateView
from django.views.generic.detail import DetailView
from django.utils.decorators import method_decorator
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import reverse, reverse_lazy
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages
from django.http import Http404

from djf_surveys.app_settings import SURVEYS_ADMIN_BASE_PATH
from djf_surveys.models import Survey, Question, TYPE_FIELD
from djf_surveys.mixin import ContextTitleMixin
from djf_surveys.admins.v2.forms import QuestionForm, QuestionWithChoicesForm, QuestionFormRatings


@method_decorator(staff_member_required, name='dispatch')
class AdminCreateQuestionView(ContextTitleMixin, CreateView):
    template_name = 'djf_surveys/admins/question_form_v2.html'
    success_url = reverse_lazy("djf_surveys:")
    title_page = _("Add Question")
    survey = None
    type_field_id = None

    def dispatch(self, request, *args, **kwargs):
        self.survey = get_object_or_404(Survey, id=kwargs['pk'])
        self.type_field_id = kwargs['type_field']
        if self.type_field_id not in TYPE_FIELD:
            raise Http404
        return super().dispatch(request, *args, **kwargs)

    def get_form_class(self):
        choices = [TYPE_FIELD.multi_select, TYPE_FIELD.select, TYPE_FIELD.radio]
        if self.type_field_id in choices:
            return QuestionWithChoicesForm
        elif self.type_field_id == TYPE_FIELD.rating:
            return QuestionFormRatings
        else:
            return QuestionForm

    def post(self, request, *args, **kwargs):
        form = self.get_form()
        if form.is_valid():
            question = form.save(commit=False)
            question.survey = self.survey
            question.type_field = self.type_field_id
            question.save()
            messages.success(self.request, gettext("%(page_action_name)s succeeded.") % dict(page_action_name=capfirst(self.title_page.lower())))
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['type_field_id'] = self.type_field_id
        return context

    def get_success_url(self):
        return reverse("djf_surveys:admin_forms_survey", args=[self.survey.slug])

    def get_sub_title_page(self):
        return gettext("Type Field %s") % Question.TYPE_FIELD[self.type_field_id][1]


@method_decorator(staff_member_required, name='dispatch')
class AdminUpdateQuestionView(ContextTitleMixin, UpdateView):
    model = Question
    template_name = 'djf_surveys/admins/question_form_v2.html'
    success_url = SURVEYS_ADMIN_BASE_PATH
    title_page = _("Edit Question")
    survey = None
    type_field_id = None

    def dispatch(self, request, *args, **kwargs):
        question = self.get_object()
        self.type_field_id = question.type_field
        self.survey = question.survey
        return super().dispatch(request, *args, **kwargs)

    def get_form_class(self):
        choices = [TYPE_FIELD.multi_select, TYPE_FIELD.select, TYPE_FIELD.radio]
        if self.type_field_id in choices:
            return QuestionWithChoicesForm
        elif self.type_field_id == TYPE_FIELD.rating:
            return QuestionFormRatings
        else:
            return QuestionForm
        
    def get_object(self):
        object = super(UpdateView, self).get_object(self.get_queryset())
        if object.type_field == TYPE_FIELD.rating:
            if object.choices == None: # use 5 as default for backward compatibility
                object.choices = 5
        return object

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['type_field_id'] = self.type_field_id
        return context

    def get_success_url(self):
        return reverse("djf_surveys:admin_forms_survey", args=[self.survey.slug])

    def get_sub_title_page(self):
        question = self.get_object()
        return gettext("Type Field %s") % question.get_type_field_display()
